#
# TV::Fetch - manage fetching and storing data
# ~~~~~~~~~   (c) Andrew Flegg 2002. Released under the Artistic Licence.

package TV::Fetch;

use TV::Core;
use Date::Calc;
use XML::Simple;
use LWP::UserAgent;
use HTML::Entities;

use vars qw($DEBUG $UA);

# -- Configuration variables ----------------------------------------
#
$DEBUG = 0;
$FORCE = 0;
$UA    = LWP::UserAgent->new(keep_alive => 4,
                             timeout => 30,
                             env_proxy => 1,
                             agent => 'Mozilla/5.0 (X11; Linux i686) Gecko');

# -- Get and store listings -----------------------------------------
#
sub store {
    my $class   = shift;
    my $channel = shift or return;
    my @days    = @_;

    my $datadir = $TV::Core::DATA_DIR;
    my $config  = XMLin("$datadir/channels/$channel.xml");
    die "Unable to read config for $file\n" unless $config;

    $config->{debug} = $DEBUG;
    my $type = $config->{source};
    my $obj  = eval("use TV::Fetch::$type;
                     return TV::Fetch::$type->new('$channel', \$config)");
    if ($@) { warn "Error with creating $obj fetcher: $@"; return; }
    return unless $obj;

    foreach my $d (@days) {
        my $file = "$d/$channel.xml";
        if ((-s "$datadir/listings/$file") && (! $FORCE)) {
            print "+++ $file exists - skipping.\n" if $DEBUG;
            next;
        }

        my $list;
        print "+++ Getting day $d...\n" if $DEBUG;
        $list = $obj->getProgrammes($d);

        my $now  = scalar(localtime());
        my @show = Date::Calc::Add_Delta_Days(Date::Calc::Today(), $d);
        my $date = substr("0$show[2]", -2).'/'.substr("0$show[1]",-2).'/'.$show[0];
        my $op   = '<?xml version="1.0" encoding="UTF-8"?>'.
                   "\n<!-- produced for the bleb.org TV system at $now -->".
                   "\n<channel id=\"$channel\" source=\"$type\" date=\"$date\">\n";

        foreach my $p (@$list) {
            next if ($p->{'start'} || 's') eq ($p->{'end'} || 'e');
            $op .= "  <programme>\n";
            while (my ($k, $v) = each %$p) {
                next unless defined($k) and defined($v);
                $v   =~ s/[\r\n]+//g;
                $v   =~ s/^\s+//g;
                $v   =~ s/\s+$//g;
                $v   = encode_entities($v, '<>&"');
                $op .= "    <$k>$v</$k>\n";
            }
            $op .= "  </programme>\n";
        }
        $op .= "</channel>\n";

        if (scalar(@$list)) {
            print "+++ $channel/$d has ".scalar(@$list)." programmes\n" if $DEBUG;
            open(OUT, '>:utf8', "$datadir/listings/$file") or die "unable to write $file: $!\n";
            print OUT $op;
            close(OUT) or die "unable to close $file: $!\n";
        } else {
            warn "+++ $channel/$d has no programmes\n";
        }
        sleep 4;
    }

    return;
}


# -- Fetch a webpage ----------------------------------------------------------
#
sub get {
    my ($url, %heads) = @_;
    
    my $req = new HTTP::Request(GET => $url);
    while (my ($k, $v) = each %heads) {
        $req->push_header($k => $v);
    }

    my $res = $UA->request($req);
    if ($res->is_error) {
        warn "unable to fetch $url (".$res->status_line.")\n";
        return undef;
    }

    return $res->content;
}


1;
